package com.link.core.contextholder;

import com.link.core.util.RedisUtil;
import com.link.core.util.dynamicredis.RedisSource;
import com.link.core.util.redisclient.LinkRedisClient;
import java.util.Objects;
import java.util.concurrent.ConcurrentLinkedQueue;
import java.util.function.Function;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.util.CollectionUtils;
import org.springframework.web.context.request.RequestContextHolder;

public class RedisSecurityContextHolder {
   private static Logger logger = LoggerFactory.getLogger(RedisSecurityContextHolder.class);
   private static final String REDIS_CONTEXT_ATTRIBUTES = "REDIS_SECURITY_CONTEXT";

   public static void set(Function<LinkRedisClient, Object> function) {
      ConcurrentLinkedQueue<Function<LinkRedisClient, Object>> functions = get();
      if (CollectionUtils.isEmpty(functions)) {
         functions = new ConcurrentLinkedQueue();
         RequestContextHolder.currentRequestAttributes().setAttribute("REDIS_SECURITY_CONTEXT", functions, 0);
      }

      functions.add(function);
   }

   private static ConcurrentLinkedQueue<Function<LinkRedisClient, Object>> get() {
      return (ConcurrentLinkedQueue)RequestContextHolder.currentRequestAttributes().getAttribute("REDIS_SECURITY_CONTEXT", 0);
   }

   public static void roolBack(RedisSource redisSource) {
      ConcurrentLinkedQueue<Function<LinkRedisClient, Object>> functions = get();
      if (!CollectionUtils.isEmpty(functions)) {
         LinkRedisClient redisClient = null;

         try {
            redisClient = RedisUtil.getJedis(redisSource);

            while(!functions.isEmpty()) {
               Function<LinkRedisClient, Object> function = (Function)functions.peek();
               Object result = function.apply(redisClient);
               functions.remove();
               logger.info("执行回滚操作:" + (Objects.isNull(result) ? "null" : result.toString()));
            }
         } catch (Exception var8) {
            Exception e = var8;
            logger.error("回滚redis错误：" + e.getMessage());
            logger.info("还没有进行回滚的操作有以下内容：");

            while(!functions.isEmpty()) {
               Function<LinkRedisClient, Object> function = (Function)functions.peek();
               logger.info("还没有执行的回滚操作：" + function.toString());
               functions.remove();
            }

            throw e;
         } finally {
            RedisUtil.returnResource(redisClient);
         }

      }
   }
}
